#!/bin/sh
#
#	Compile a single executable from three .c files in the
#	local directory.  Update one of the files and re-compile.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [selective module update]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >Construct			# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'foo', qw (
	a.c
	b.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >a.c
a()
{
	printf("In function a().\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >b.c
b()
{
	printf("In function b().\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >main.c
main()
{
	a();
	b();
	printf("SUCCESS!\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
./foo > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
In function a().
In function b().
SUCCESS!
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

#
cat <<'_EOF_' >b.c
b()
{
	printf("This is function b() the second.\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . > cons.2.out 2>&1
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
./foo > foo.out
if test $? -ne 0 ; then fail 2 ; fi
diff - foo.out <<'_EOF_'
In function a().
This is function b() the second.
SUCCESS!
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

#
pass
