#!/bin/sh
#
#	Install a file from one subdirectory to another.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Install]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir src
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<'_EOF_' >Construct
Build 'src/Conscript';
_EOF_
cat <<_EOF_ >src/Conscript			# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Install \$env '#install', 'foo.sh';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >src/foo.sh
#!/bin/sh
echo "Called with \$0 = '$0'"
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
sh install/foo.sh > foo.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.out <<'_EOF_'
Called with $0 = 'install/foo.sh'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

#
pass
