#!/bin/sh
#
#	Define two executables to be built from duplicate .c files.
#	Define a built environment with CFLAGS dependent on %COPT.
#	Clone the environment, changing only %COPT.  Built one
#	executable with one environment and the other with the
#	other, making sure that each executable used the correct
#	%COPT value for its environment.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [clone]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >Construct			# NO SINGLE QUOTES, EXPAND $CC
\$env1 = new cons (
	CC	=> '$CC',
	COPT	=> '-DENV=1',
	CFLAGS	=> '-DSTRING=\"FOO\" %COPT',
);
\$env2 = \$env1->clone(
	COPT	=> '-DENV=2',
);
Program \$env1 'foo1', 'foo1.c';
Program \$env2 'foo2', 'foo2.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo1.c
main()
{
	printf("This is the foo[12].c file, ENV = %d:  %s.\n", ENV, STRING);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cp foo1.c foo2.c
if test $? -ne 0 ; then no_result ; fi

$prog . >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
./foo1 > foo1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo1.out <<'_EOF_'
This is the foo[12].c file, ENV = 1:  FOO.
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
./foo2 > foo2.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo2.out <<'_EOF_'
This is the foo[12].c file, ENV = 2:  FOO.
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

#
pass
