#!/bin/sh
#
#	Create Construct and Conscript files for creating four
#	executables, two from .c files in the local directory and
#	two from .c files in a subdirectory.  Invoke cons -pa,
#	compare the output against expected results.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-pa]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir subdir
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >Construct				# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'a', 'a.c';
Program \$env 'b', 'b.c';
Build 'subdir/Conscript';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >a.c
main()
{
	printf("a.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >b.c
main()
{
	printf("b.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<_EOF_ >subdir/Conscript			# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'c', 'c.c';
Program \$env 'd', 'd.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/c.c
main()
{
	printf("subdir/c.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >subdir/d.c
main()
{
	printf("subdir/d.c\n");
	exit (0);
}
_EOF_

$prog -pa . >cons.1.out 2>cons.1.err
if test $? -ne 0 ; then fail 1 cons.1.err ; fi
diff - cons.1.out <<_EOF_			# NO SINGLE QUOTES, EXPAND $CC
a:
... $CC -o a a.o
a.o:
... $CC -c a.c -o a.o
b:
... $CC -o b b.o
b.o:
... $CC -c b.c -o b.o
subdir/c:
... $CC -o subdir/c subdir/c.o
subdir/c.o:
... $CC -c subdir/c.c -o subdir/c.o
subdir/d:
... $CC -o subdir/d subdir/d.o
subdir/d.o:
... $CC -c subdir/d.c -o subdir/d.o
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
diff - cons.1.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
if test -s a ; then fail 1 ; fi
if test -s b ; then fail 1 ; fi
if test -s subdir/c ; then fail 1 ; fi
if test -s subdir/d ; then fail 1 ; fi

#
pass
