#!/bin/sh
#
#	Echo all combinations of %[<>0-9](:[df]?)? variables into
#	various files.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [%-variables]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir sub
if test $? -ne 0 ; then no_result ; fi
mkdir sub/dir
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >Construct				# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
@deps = qw(
	foo_1
	foo_2
	foo_3
	sub/foo_4
	sub/foo_5
	sub/foo_6
	sub/dir/foo_7
	sub/dir/foo_8
	sub/dir/foo_9
	foo_10
);
Command \$env 'out_left', @deps, qq(
	echo '========== <' >> %>
	echo "< is '%<'" >> %>
	echo "<:d is '%<:d'" >> %>
	echo "<:f is '%<:f'" >> %>
);
Command \$env 'out_right', @deps, qq(
	echo '========== >' >> %>
	echo "> is '%>'" >> %>
	echo ">:d is '%>:d'" >> %>
	echo ">:f is '%>:f'" >> %>
);
Command \$env 'sub/out_zero', @deps, qq(
	echo '========== 0' >> %>
	echo "0 is '%0'" >> %>
	echo "0:d is '%0:d'" >> %>
	echo "0:f is '%0:f'" >> %>
);
Command \$env 'sub/dir/out_right', @deps, qq(
	echo '========== >' >> %>
	echo "> is '%>'" >> %>
	echo ">:d is '%>:d'" >> %>
	echo ">:f is '%>:f'" >> %>
);
Command \$env 'out_numbers', @deps, qq(
	echo '========== 1' >> %>
	echo "1 is '%1'" >> %>
	echo "1:d is '%1:d'" >> %>
	echo "1:f is '%1:f'" >> %>
	echo '========== 2' >> %>
	echo "2 is '%2'" >> %>
	echo "2:d is '%2:d'" >> %>
	echo "2:f is '%2:f'" >> %>
	echo '========== 3' >> %>
	echo "3 is '%3'" >> %>
	echo "3:d is '%3:d'" >> %>
	echo "3:f is '%3:f'" >> %>
	echo '========== 4' >> %>
	echo "4 is '%4'" >> %>
	echo "4:d is '%4:d'" >> %>
	echo "4:f is '%4:f'" >> %>
	echo '========== 5' >> %>
	echo "5 is '%5'" >> %>
	echo "5:d is '%5:d'" >> %>
	echo "5:f is '%5:f'" >> %>
	echo '========== 6' >> %>
	echo "6 is '%6'" >> %>
	echo "6:d is '%6:d'" >> %>
	echo "6:f is '%6:f'" >> %>
	echo '========== 7' >> %>
	echo "7 is '%7'" >> %>
	echo "7:d is '%7:d'" >> %>
	echo "7:f is '%7:f'" >> %>
	echo '========== 8' >> %>
	echo "8 is '%8'" >> %>
	echo "8:d is '%8:d'" >> %>
	echo "8:f is '%8:f'" >> %>
	echo '========== 9' >> %>
	echo "9 is '%9'" >> %>
	echo "9:d is '%9:d'" >> %>
	echo "9:f is '%9:f'" >> %>
);
Command \$env 'out_list', @deps, qq(
	echo '========== 1 3 5 7 9 <' >> %>
	echo %1 %3 %5 %7 %9 > /dev/null; echo "after 1 3 5 7 9, < is '%<'" >> %>
	echo %1 %3 %5 %7 %9 > /dev/null; echo "after 1 3 5 7 9, <:f is '%<:f'" >> %>
	echo %1 %3 %5 %7 %9 > /dev/null; echo "after 1 3 5 7 9, <:d is '%<:d'" >> %>
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo_1
This is the foo_1 file.
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo_2
This is the foo_2 file.
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo_3
This is the foo_3 file.
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >sub/foo_4
This is the sub/foo_4 file.
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >sub/foo_5
This is the sub/foo_5 file.
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >sub/foo_6
This is the sub/foo_6 file.
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >sub/dir/foo_7
This is the sub/dir/foo_7 file.
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >sub/dir/foo_8
This is the sub/dir/foo_8 file.
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >sub/dir/foo_9
This is the sub/dir/foo_9 file.
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >foo_10
This is the foo_10 file.
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog . >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - out_left <<'_EOF_'
========== <
< is 'foo_1 foo_2 foo_3 sub/foo_4 sub/foo_5 sub/foo_6 sub/dir/foo_7 sub/dir/foo_8 sub/dir/foo_9 foo_10'
<:d is '. . . sub sub sub sub/dir sub/dir sub/dir .'
<:f is 'foo_1 foo_2 foo_3 foo_4 foo_5 foo_6 foo_7 foo_8 foo_9 foo_10'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
diff - out_right <<'_EOF_'
========== >
> is 'out_right'
>:d is '.'
>:f is 'out_right'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
diff - sub/out_zero <<'_EOF_'
========== 0
0 is 'sub/out_zero'
0:d is 'sub'
0:f is 'out_zero'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
diff - sub/dir/out_right <<'_EOF_'
========== >
> is 'sub/dir/out_right'
>:d is 'sub/dir'
>:f is 'out_right'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
diff - out_numbers <<'_EOF_'
========== 1
1 is 'foo_1'
1:d is '.'
1:f is 'foo_1'
========== 2
2 is 'foo_2'
2:d is '.'
2:f is 'foo_2'
========== 3
3 is 'foo_3'
3:d is '.'
3:f is 'foo_3'
========== 4
4 is 'sub/foo_4'
4:d is 'sub'
4:f is 'foo_4'
========== 5
5 is 'sub/foo_5'
5:d is 'sub'
5:f is 'foo_5'
========== 6
6 is 'sub/foo_6'
6:d is 'sub'
6:f is 'foo_6'
========== 7
7 is 'sub/dir/foo_7'
7:d is 'sub/dir'
7:f is 'foo_7'
========== 8
8 is 'sub/dir/foo_8'
8:d is 'sub/dir'
8:f is 'foo_8'
========== 9
9 is 'sub/dir/foo_9'
9:d is 'sub/dir'
9:f is 'foo_9'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi
diff - out_list <<'_EOF_'
========== 1 3 5 7 9 <
after 1 3 5 7 9, < is 'foo_2 sub/foo_4 sub/foo_6 sub/dir/foo_8 foo_10'
after 1 3 5 7 9, <:f is 'foo_2 foo_4 foo_6 foo_8 foo_10'
after 1 3 5 7 9, <:d is '. sub sub sub/dir .'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

#
pass
