#!/bin/sh
#
#	Define a Construct file to compile a single executable from
#	a single .c file in the	local directory, with Help text
#	defined.  Check that the help text is generated by -h.
#	Check that the Program builds successfully without -h.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [single-module Program]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >Construct			# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'foo', 'foo.c';
Help qq(
This is help text.
It should appear in response to a -h flag.
);
_EOF_
if test $? -ne 0 ; then no_result ; fi

cat <<'_EOF_' >foo.c
main()
{
	printf("SUCCESS!\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

$prog -h >cons.1.out 2>cons.1.err
if test $? -ne 2 ; then fail 1 cons.1.err ; fi	# NOTE FAILURE, -ne 2
diff - cons.1.out <<'_EOF_'

This is help text.
It should appear in response to a -h flag.

_EOF_
if test $? -ne 0 ; then fail 1 ; fi
diff - cons.1.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

$prog . >cons.2.out 2>&1
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
./foo > foo.out
if test $? -ne 0 ; then fail 2 ; fi
diff - foo.out <<'_EOF_'
SUCCESS!
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

#
pass
