#!/bin/sh
#
#	Build a library from two modules in a repository directory.
#	Link the library to another modules from the repository
#	directory to generate an executable.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Library, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >repository/Construct # NO SINGLE QUOTES, EXPAND $CC, $AR, $RANLIB
\$libenv = new cons (
	AR	=> '$AR',
	CC	=> '$CC',
	RANLIB	=> '$RANLIB',
);
Library \$libenv 'libfoo', qw (
	a.c
	b.c
);
\$fooenv = new cons(
	AR	=> '$AR',
	CC	=> '$CC',
	LIBPATH => [ '.' ],
	LIBS	=> '-lfoo',
	RANLIB	=> '$RANLIB',
);
Program \$fooenv 'foo', 'main.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/main.c
main()
{
	a();
	b();
	printf("repository/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/a.c
a()
{
	printf("repository/a.c\n");
}
_EOF_
cat <<'_EOF_' >repository/b.c
b()
{
	printf("repository/b.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
work/foo > foo.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.1.out <<'_EOF_'
repository/a.c
repository/b.c
repository/main.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

#
pass
