#!/bin/sh
#
#	Compile a single executable from three .c files in a
#	repository subdirectory.   Compilation takes place in a
#	separate local build subdirectory established via the 'Link'
#	command, pulling the .c files directly from the repository
#	subdirectory.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Link, -R]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir work/src
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi
mkdir repository/src
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<'_EOF_' >repository/Construct
Link 'build' => 'src';
Build qw(
	build/Conscript
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<_EOF_ >repository/src/Conscript		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'foo', qw (
	a.c
	b.c
	main.c
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/a.c
a()
{
	printf("repository/src/a.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/b.c
b()
{
	printf("repository/src/b.c\n");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/src/main.c
main()
{
	a();
	b();
	printf("repository/src/main.c\n");
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
work/build/foo > foo.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.1.out <<'_EOF_'
repository/src/a.c
repository/src/b.c
repository/src/main.c
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

#
pass
