#!/bin/sh
#
#	Create subdirectories work, foo, and bar.  Construct file
#	specifies a CPPPATH of .:zzz, a Prgoram that builds a single
#	.c file, and a Command that echoes %CCCOM to an output
#	file.  In the work directory, invoke cons -R foo -R work
#	-R bar.  Examine the CCCOM output to make sure that the -I
#	flags properly exclude the work subdirectory in which we
#	executed.
#
#	NOTE:  THIS TEST EXAMINES THE ACTIONS USED TO BUILD FILES.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [-R, strip current directory]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir foo
if test $? -ne 0 ; then no_result ; fi
mkdir bar
if test $? -ne 0 ; then no_result ; fi
mkdir work
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >work/Construct		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons (
	CC	=> '$CC',
	CPPPATH => '.:zzz',
);
Program \$env 'foo', 'foo.c';
Command \$env 'CCCOM.out', 'foo', qq(
	echo %CCCOM > %>
);
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >work/foo.c
main()
{
	printf("work/foo.c");
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(cd work; $prog -R $work/foo -R $work/work -R $work/bar .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - work/CCCOM.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $CC, $work
$CC -I. -I$work/foo -I$work/bar -Izzz -I$work/foo/zzz -I$work/bar/zzz -c foo -o CCCOM.out
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

#
pass
