#!/bin/sh
#
#	Create an executable from a .c file in the repository.
#	Build in a work subdirectory; everything is up-to-date.
#	Create a work Construct file that specifies the executable
#	is Local.  Build; the repository executable should be copied
#	into the work directory.  Create a work .c file; build;
#	the work executable should be updated.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Local]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir work
if test $? -ne 0 ; then no_result ; fi
mkdir repository
if test $? -ne 0 ; then no_result ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >repository/Construct	# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'foo', 'foo.c';
_EOF_
if test $? -ne 0 ; then no_result ; fi
cat <<'_EOF_' >repository/foo.c
main(int argc, char *argv[])
{
	printf("repository/foo.c called with arg 0 = '%s'\n", *argv);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd repository && $prog .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
repository/foo > foo.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.1.out <<'_EOF_'
repository/foo.c called with arg 0 = 'repository/foo'
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

# Make the repository non-writable,
# so we'll detect if we try to write into it accidentally.
find repository -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.err ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<_EOF_
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
if test -s work/foo.o ; then fail 2 ; fi
if test -s work/foo ; then fail 2 ; fi

cat <<_EOF_ >work/Construct		# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'foo', 'foo.c';
Local 'foo';
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.3.out 2>&1
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
work/foo > foo.3.out
if test $? -ne 0 ; then fail 3 ; fi
diff - foo.3.out <<'_EOF_'
repository/foo.c called with arg 0 = 'work/foo'
_EOF_
if test $? -ne 0 ; then fail 3 ; fi
if test -s work/foo.o ; then fail 3 ; fi

cat <<'_EOF_' >work/foo.c
main(int argc, char *argv[])
{
	printf("work/foo.c called with arg 0 = '%s'\n", *argv);
	exit (0);
}
_EOF_
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog -R $work/repository .) >cons.4.out 2>&1
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
work/foo > foo.4.out
if test $? -ne 0 ; then fail 4 ; fi
diff - foo.4.out <<'_EOF_'
work/foo.c called with arg 0 = 'work/foo'
_EOF_
if test $? -ne 0 ; then fail 4 ; fi

#
pass
