#!/bin/sh
#
#	Create four empty repository directories, and a work
#	Construct file that specifies Repository for two of them,
#	and then fetches the Repository_List and prints it to a
#	file.  Invoke cons; check that the list was printed correctly.
#	Invoke cons -R -R, check that the argument repositories
#	preced the Construct file repositories.  Re-create the
#	Construct file without the Repository method.  Invoke cons
#	-R -R -R, check that the repository list is correct despite
#	the lack of the Repository method.  Invoke cons without any
#	-R flags, check that the output lists no repositories.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [Repository_List]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	cd $here
	find $work -type d -user $USER -exec chmod u+w {} \;
	if test X$PRESERVE = X -a X$1 = X ; then
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test$teststr of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

mkdir repository.1
if test $? -ne 0 ; then exit 2 ; fi
mkdir repository.2
if test $? -ne 0 ; then exit 2 ; fi
mkdir repository.3
if test $? -ne 0 ; then exit 2 ; fi
mkdir repository.4
if test $? -ne 0 ; then exit 2 ; fi
mkdir work
if test $? -ne 0 ; then exit 2 ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<_EOF_ >work/Construct		# NO SINGLE QUOTES, EXPAND $CC, $work
\$env = new cons ( CC => '$CC' );
Repository qw (
	$work/repository.2
	$work/repository.1
);
@list = Repository_List;
\$rep_list_str = join(' ', @list);
\$repository_out = "repository_list.\$ARG{TEST}.out";
Command \$env \$repository_out, 'Construct', qq(
	echo "\$rep_list_str" > %>
);
_EOF_
if test $? -ne 0 ; then no_result ; fi

# Make the repositories non-writable,
# so we'll detect if we try to write into them accidentally.
find repository.1 -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi
find repository.2 -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi
find repository.3 -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi
find repository.4 -user $USER -exec chmod u-w {} \;
if test $? -ne 0 ; then no_result ; fi

(set +x; cd work && $prog TEST=1 .) >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
diff - work/repository_list.1.out <<_EOF_ # NO SINGLE QUOTES, EXPAND $work
$work/repository.2 $work/repository.1
_EOF_
if test $? -ne 0 ; then fail 1 ; fi

(set +x; cd work && $prog TEST=2 -R $work/repository.4 -R $work/repository.3 .) >cons.2.out 2>&1
if test $? -ne 0 ; then fail 2 cons.2.out ; fi
diff - work/repository_list.2.out <<_EOF_ # NO SINGLE QUOTES, EXPAND $work
$work/repository.4 $work/repository.3 $work/repository.2 $work/repository.1
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

cat <<_EOF_ >work/Construct		# NO SINGLE QUOTES, EXPAND $CC, $work
\$env = new cons ( CC => '$CC' );
@list = Repository_List;
\$rep_list_str = join(' ', @list);
\$repository_out = "repository_list.\$ARG{TEST}.out";
Command \$env \$repository_out, 'Construct', qq(
	echo "\$rep_list_str" > %>
);
_EOF_

(set +x; cd work && $prog TEST=3 -R $work/repository.4 -R $work/repository.1 -R $work/repository.3 .) >cons.3.out 2>&1
if test $? -ne 0 ; then fail 3 cons.3.out ; fi
diff - work/repository_list.3.out <<_EOF_ # NO SINGLE QUOTES, EXPAND $work
$work/repository.4 $work/repository.1 $work/repository.3
_EOF_
if test $? -ne 0 ; then fail 3 ; fi

(set +x; cd work && $prog TEST=4 .) >cons.4.out 2>&1
if test $? -ne 0 ; then fail 4 cons.4.out ; fi
diff - work/repository_list.4.out <<_EOF_ # NO SINGLE QUOTES, EXPAND $work

_EOF_
if test $? -ne 0 ; then fail 4 ; fi

#
pass
