#!/bin/sh
#
#	DESCRIBE HOW THE FUNCTIONALITY IS TESTED,
#	AND ADD THIS DESCRIPTION TO THE Tests.doc FILE.
#

work=${TMPDIR:-/tmp}/$$

here=`pwd`
if test $? -ne 0 ; then exit 2 ; fi

prog=${PROG:-cons}
progstr="$prog [DESCRIPTION_OF_FUNCTIONALITY_BEING_TESTED]"
case $prog in
/* ) ;;
* ) prog=$here/$prog ;;
esac

cleanup()
{
	if test X$PRESERVE = X -a X$1 = X ; then
		cd $here
		find $work -type d -user $USER -exec chmod u+w {} \;
		rm -rf $work
	else
		echo Preserving work directory $work
	fi
}

no_result()
{
	set +x
	echo NO RESULT for test of $progstr 1>&2
	cleanup $PRESERVE_NO_RESULT
	exit 2
}
fail()
{
	set +x
	if test "$1" ; then teststr=" #$1" ; fi
	if test "$2" ; then
		echo FAILED test$teststr of $progstr: 1>&2
		cat $2 1>&2
	fi
	echo FAILED test of $progstr 1>&2
	cleanup $PRESERVE_FAIL
	exit 1
}
pass()
{
	set +x
	echo PASSED 1>&2
	cleanup $PRESERVE_PASS
	exit 0
}
trap "no_result" 1 2 3 15

#
# make the work directory
#
mkdir $work
if test $? -ne 0 ; then exit 2 ; fi
cd $work
if test $? -ne 0 ; then exit 2 ; fi

#
AR=${AR:-ar}
CC=${CC:-cc}
RANLIB=${RANLIB:-ranlib}

#
cat <<EOF >Conscript			# NO SINGLE QUOTES, EXPAND $CC
\$env = new cons ( CC => '$CC' );
Program \$env 'foo', 'foo.c';
EOF
if test $? -ne 0 ; then no_result ; fi
cat <<EOF >foo.c
main()
{
	printf("foo.c\n");
	exit (0);
}
EOF
if test $? -ne 0 ; then no_result ; fi

# Canonical execution of Cons to build something:
# success or failure is derived from the exit code,
# combined stdout and stderr output is displayed on failure.
# On success, the generated program is executed
# and its output checked.
$prog . >cons.1.out 2>&1
if test $? -ne 0 ; then fail 1 cons.1.out ; fi
./foo >foo.1.out
if test $? -ne 0 ; then fail 1 ; fi
diff - foo.1.out <<EOF
foo.c
EOF
if test $? -ne 0 ; then fail 1 ; fi

# Canonical execution of Cons when checking output of Cons itself:
# stderr output is displayed on failure,
# stdout and stderr output are checked separately on success.
$prog . >cons.2.out 2>cons.2.err
if test $? -ne 0 ; then fail 2 cons.2.err ; fi
diff - cons.2.out <<_EOF_		# NO SINGLE QUOTES, EXPAND $prog
`basename $prog`: "." is up-to-date.
_EOF_
if test $? -ne 0 ; then fail 2 ; fi
diff - cons.2.err <<'_EOF_'
_EOF_
if test $? -ne 0 ; then fail 2 ; fi

#
pass
